@@ -5,11 +5,11 @@ use warnings;
 use Carp;
 use FFI::Platypus::Buffer;
 
-our $VERSION = 0.0601;
+our $VERSION = 0.0700;
 
 sub DESTROY {
     my $self = shift;
-    delete $Geo::GDAL::FFI::parent{$$self};
+    Geo::GDAL::FFI::_deregister_parent_ref ($$self);
 }
 
 sub GetDataType {
