@@ -4,8 +4,9 @@ use strict;
 use warnings;
 use Carp;
 use base 'Geo::GDAL::FFI::Object';
+use Scalar::Util qw /blessed/;
 
-our $VERSION = 0.0601;
+our $VERSION = 0.0700;
 
 sub DESTROY {
     my $self = shift;
@@ -78,7 +79,7 @@ sub GetBand {
     my ($self, $i) = @_;
     $i //= 1;
     my $b = Geo::GDAL::FFI::GDALGetRasterBand($$self, $i);
-    $Geo::GDAL::FFI::parent{$b} = $self;
+    Geo::GDAL::FFI::_register_parent_ref ($b, $self);
     return bless \$b, 'Geo::GDAL::FFI::Band';
 }
 
@@ -95,8 +96,8 @@ sub GetLayer {
     my ($self, $i) = @_;
     $i //= 0;
     my $l = Geo::GDAL::FFI::isint($i) ? Geo::GDAL::FFI::GDALDatasetGetLayer($$self, $i) :
-        Geo::GDAL::FFI::GDALDatasetGetLayerByName($$self, $i);
-    $Geo::GDAL::FFI::parent{$l} = $self;
+    Geo::GDAL::FFI::GDALDatasetGetLayerByName($$self, $i);
+    Geo::GDAL::FFI::_register_parent_ref ($l, $self);
     return bless \$l, 'Geo::GDAL::FFI::Layer';
 }
 
@@ -124,7 +125,7 @@ sub CreateLayer {
     Geo::GDAL::FFI::OSRRelease($sr) if $sr;
     my $msg = Geo::GDAL::FFI::error_msg();
     confess $msg if $msg;
-    $Geo::GDAL::FFI::parent{$l} = $self;
+    Geo::GDAL::FFI::_register_parent_ref ($l, $self);
     my $layer = bless \$l, 'Geo::GDAL::FFI::Layer';
     if (exists $args->{Fields}) {
         for my $f (@{$args->{Fields}}) {
@@ -152,10 +153,35 @@ sub CopyLayer {
         my $msg = Geo::GDAL::FFI::error_msg() // "GDALDatasetCopyLayer failed.";
         confess $msg if $msg;
     }
-    $Geo::GDAL::FFI::parent{$l} = $self;
+    Geo::GDAL::FFI::_register_parent_ref ($l, $self);
     return bless \$l, 'Geo::GDAL::FFI::Layer';
 }
 
+
+sub ExecuteSQL {
+    my ($self, $sql, $filter, $dialect) = @_;
+        
+    my $lyr = Geo::GDAL::FFI::GDALDatasetExecuteSQL(
+        $$self, $sql, $$filter, $dialect
+    );
+    
+    if ($lyr) {
+        if (defined wantarray) {
+            Geo::GDAL::FFI::_register_parent_ref ($lyr, $self);
+            return bless \$lyr, 'Geo::GDAL::FFI::Layer::ResultSet';
+        }
+        else {
+            Geo::GDAL::FFI::GDALDatasetReleaseResultSet ($lyr, $$self);            
+        }
+    }
+
+    #  This is perhaps unnecessary, but ensures
+    #  internal  details do not leak if spatial
+    #  index is built in non-void context.
+    return undef;
+}
+
+
 ## utilities
 
 sub new_options {
@@ -216,38 +242,64 @@ sub destination {
 sub dataset_input {
     my ($self, $input) = @_;
     $input //= [];
-    confess "The input must be a reference to an array of datasets." unless ref $input;
+    confess "The input must be a reference to an array of datasets." unless ref ($input) =~ /ARRAY/;
     my @datasets = ($$self);
-    for my $dataset (@$input) {
-        push @datasets, $$input;
+    for my $ds (@$input) {
+        push @datasets, $$ds;
     }
     return \@datasets;
 }
 
 sub Warp {
     my ($self, $args) = @_;
+    
     my ($path, $dst) = destination($args->{Destination});
+    confess "Destination object should not be passed for non-void context"
+      if defined wantarray && blessed $dst;
+
     my $input = $self->dataset_input($args->{Input});
+
     my $options = new_options(\&Geo::GDAL::FFI::GDALWarpAppOptionsNew, $args->{Options});
     set_progress($options, $args, \&Geo::GDAL::FFI::GDALWarpAppOptionsSetProgress);
+    
     my $e = 0;
-    my $warped = Geo::GDAL::FFI::GDALWarp($path, $dst, scalar @$input, $input, $options, \$e);
+    my $result;
+    if (blessed($dst)) {
+        Geo::GDAL::FFI::GDALWarp($path, $dst, scalar @$input, $input, $options, \$e);
+    } else {
+        $result = Geo::GDAL::FFI::GDALWarp($path, undef, scalar @$input, $input, $options, \$e);
+    }
     Geo::GDAL::FFI::GDALWarpAppOptionsFree($options);
-    confess Geo::GDAL::FFI::error_msg() // 'Warp failed.' if !$warped || $e != 0;
-    return bless \$warped, 'Geo::GDAL::FFI::Dataset';
+    if (defined $result) {
+        confess Geo::GDAL::FFI::error_msg() // 'Warp failed.' if !$result || $e != 0;
+        return bless \$result, 'Geo::GDAL::FFI::Dataset';
+    }
 }
 
 sub VectorTranslate {
     my ($self, $args) = @_;
     my ($path, $dst) = destination($args->{Destination});
+    confess "Destination object should not be passed for non-void context"
+      if defined wantarray && blessed $dst;
+
     my $input = $self->dataset_input($args->{Input});
+
     my $options = new_options(\&Geo::GDAL::FFI::GDALVectorTranslateOptionsNew, $args->{Options});
     set_progress($options, $args, \&Geo::GDAL::FFI::GDALVectorTranslateOptionsSetProgress);
+    
     my $e = 0;
-    my $result = Geo::GDAL::FFI::GDALVectorTranslate($path, $dst, scalar @$input, $input, $options, \$e);
+    my $result;
+    if (blessed($dst)) {
+        Geo::GDAL::FFI::GDALVectorTranslate(undef, $$dst, scalar @$input, $input, $options, \$e);
+    }
+    else {
+        my $result = Geo::GDAL::FFI::GDALVectorTranslate($path, undef, scalar @$input, $input, $options, \$e);
+    }
     Geo::GDAL::FFI::GDALVectorTranslateOptionsFree($options);
-    confess Geo::GDAL::FFI::error_msg() // 'VectorTranslate failed.' if !$result || $e != 0;
-    return bless \$result, 'Geo::GDAL::FFI::Dataset';
+    confess Geo::GDAL::FFI::error_msg() // 'VectorTranslate failed.' if $e != 0;
+    if (defined $result) {
+        return bless \$result, 'Geo::GDAL::FFI::Dataset';
+    }
 }
 
 sub DEMProcessing {
@@ -265,14 +317,30 @@ sub DEMProcessing {
 
 sub NearBlack {
     my ($self, $args) = @_;
+    
     my ($path, $dst) = destination($args->{Destination});
-    my $options = new_options(\&Geo::GDAL::FFI::GDALNearBlackOptionsNew, $args->{Options});
-    set_progress($options, $args, \&Geo::GDAL::FFI::GDALNearBlackOptionsSetProgress);
+    confess "Destination object should not be passed for non-void context"
+      if defined wantarray && blessed $dst;
+
+    my $input = $self->dataset_input($args->{Input});
+
+    my $options = new_options(\&Geo::GDAL::FFI::GDALNearblackOptionsNew, $args->{Options});
+    set_progress($options, $args, \&Geo::GDAL::FFI::GDALNearblackOptionsSetProgress);
+    
     my $e = 0;
-    my $result = Geo::GDAL::FFI::GDALNearBlack($path, $dst, $$self, $options, \$e);
-    Geo::GDAL::FFI::GDALNearBlackOptionsFree($options);
-    confess Geo::GDAL::FFI::error_msg() // 'NearBlack failed.' if !$result || $e != 0;
-    return bless \$result, 'Geo::GDAL::FFI::Dataset';
+    my $result;
+    if (blessed($dst)) {
+        Geo::GDAL::FFI::GDALNearblack($path, $$dst, $$self, $options, \$e);
+    } else {
+        $result = Geo::GDAL::FFI::GDALNearblack($path, undef, $$self, $options, \$e);
+    }
+    Geo::GDAL::FFI::GDALNearblackOptionsFree($options);
+
+    confess Geo::GDAL::FFI::error_msg() // 'NearBlack failed.' if $e != 0;
+    if (defined $result) {
+        return bless \$result, 'Geo::GDAL::FFI::Dataset';
+    }
+
 }
 
 sub Grid {
@@ -289,14 +357,27 @@ sub Grid {
 
 sub Rasterize {
     my ($self, $args) = @_;
-    my ($path, $dst) = destination($args->{Destination});
+    
+    my $dst = $args->{Destination};
+    confess "Destination argument should not be passed for non-void context"
+      if defined wantarray && blessed $dst;
+
     my $options = new_options(\&Geo::GDAL::FFI::GDALRasterizeOptionsNew, $args->{Options});
     set_progress($options, $args, \&Geo::GDAL::FFI::GDALRasterizeOptionsSetProgress);
+    
     my $e = 0;
-    my $result = Geo::GDAL::FFI::GDALRasterize($path, $dst, $$self, $options, \$e);
+    my $result;
+    if (blessed($dst)) {
+        Geo::GDAL::FFI::GDALRasterize(undef, $$dst, $$self, $options, \$e);
+    } else {
+        $result = Geo::GDAL::FFI::GDALRasterize($dst, undef, $$self, $options, \$e);
+    }
     Geo::GDAL::FFI::GDALRasterizeOptionsFree($options);
-    confess Geo::GDAL::FFI::error_msg() // 'Rasterize failed.' if !$result || $e != 0;
-    return bless \$result, 'Geo::GDAL::FFI::Dataset';
+    
+    confess Geo::GDAL::FFI::error_msg() // 'Rasterize failed.' if $e != 0;
+    if (defined $result) {
+        return bless \$result, 'Geo::GDAL::FFI::Dataset';
+    }
 }
 
 sub BuildVRT {
@@ -313,6 +394,24 @@ sub BuildVRT {
 
 1;
 
+{
+    #  dummy class for result sets from ExecuteSQL
+    #  allows specialised destroy method
+    package Geo::GDAL::FFI::Layer::ResultSet;
+    use base qw /Geo::GDAL::FFI::Layer/;
+    
+    sub DESTROY {
+        my ($self) = @_;
+        my $parent = Geo::GDAL::FFI::_get_parent_ref ($$self);
+        Geo::GDAL::FFI::GDALDatasetReleaseResultSet ($$parent, $$self);
+        Geo::GDAL::FFI::_deregister_parent_ref ($$self);
+    }
+    
+    1;
+}
+
+
+
 =pod
 
 =encoding UTF-8
@@ -420,20 +519,34 @@ arguments returns the first layer.
 Copies the given layer into this dataset using the name $name and
 returns the new layer. The options hash is mostly driver specific.
 
+=head2 ExecuteSQL
+ $dataset->ExecuteSQL ($sql, $filter, $dialect);
+
+ #  build a spatial index
+ $dataset->ExecuteSQL (qq{CREATE SPATIAL INDEX ON "$some_layer_name"});
+ 
+ #  filter a data set using the SQLite dialect and a second geometry
+ my $filtered = $dataset->ExecuteSQL (
+   qq{SELECT "$fld1", "$fld2" FROM "$some_layer_name"},
+   $some_geometry,
+   'SQLite',
+ );
+ 
 =head2 Info
 
  my $info = $dataset->Info($options);
+ my $info = $dataset->Info(['-json', '-stats']);
 
 This is the same as gdalinfo utility. $options is a reference to an
-array.
+array.  Valid options are as per the L<gdalinfo|https://www.gdal.org/gdalinfo.html> utility.
 
 =head2 Translate
 
  my $target = $source->Translate($path, $options, $progress, $progress_data);
 
 Convert a raster dataset into another raster dataset. This is the same
-as the gdal_translate utility. $name is the name of the target
-dataset. $options is a reference to an array.
+as the L<gdal_translate|https://www.gdal.org/gdal_translate.html> utility. $name is the name of the target
+dataset. $options is a reference to an array of switches.
 
 =head2 Warp
 
@@ -442,6 +555,8 @@ dataset. $options is a reference to an array.
 $args is a hashref, keys may be Destination, Input, Options, Progress,
 ProgressData.
 
+Valid options are as per the L<gdalwarp|https://www.gdal.org/gdalwarp.html> utility.
+
 =head2 VectorTranslate
 
  my $result = $dataset->VectorTranslate($args);
@@ -449,6 +564,8 @@ ProgressData.
 $args is a hashref, keys may be Destination, Input, Options, Progress,
 ProgressData.
 
+Valid options are as per the L<ogr2ogr|https://www.gdal.org/ogr2ogr.html> utility.  
+
 =head2 DEMProcessing
 
  my $result = $dataset->DEMProcessing($path, $args);
@@ -456,6 +573,8 @@ ProgressData.
 $args is a hashref, keys may be Processing, ColorFilename, Options,
 Progress, ProgressData.
 
+See also L<gdaldem|https://www.gdal.org/gdaldem.html>.
+
 =head2 NearBlack
 
  my $result = $dataset->NearBlack($args);
@@ -463,17 +582,24 @@ Progress, ProgressData.
 $args is a hashref, keys may be Destination, Options, Progress,
 ProgressData.
 
+Valid options are as per the L<nearblack|https://www.gdal.org/nearblack.html> utility.  
+
 =head2 Grid
 
  my $result = $dataset->Grid($path, $options, $progress, $progress_data);
+ 
+Valid options are as per the L<gdal_grid|https://www.gdal.org/gdal_grid.html> utility.  
 
 =head2 Rasterize
 
  my $result = $dataset->Rasterize($args);
+ my $result = $dataset->Rasterize({Options => [-b => 1, -at]});
 
 $args is a hashref, keys may be Destination, Options, Progress,
 ProgressData.
 
+Valid options are as per the L<gdal_rasterize|https://www.gdal.org/gdal_rasterize.html> utility.  
+
 =head2 BuildVRT
 
  my $result = $dataset->BuildVRT($path, $args);
