@@ -4,7 +4,7 @@ use strict;
 use warnings;
 use Carp;
 
-our $VERSION = 0.0601;
+our $VERSION = 0.0700;
 
 my %ref;
 
@@ -38,7 +38,7 @@ sub new {
 
 sub DESTROY {
     my ($self) = @_;
-    delete $Geo::GDAL::FFI::parent{$$self};
+    Geo::GDAL::FFI::_deregister_parent_ref ($$self);
     if ($ref{$$self}) {
         delete $ref{$$self};
         return;
@@ -173,7 +173,7 @@ sub GetGeometryCount {
 sub GetGeometry {
     my ($self, $i) = @_;
     my $g = Geo::GDAL::FFI::OGR_G_GetGeometryRef($$self, $i);
-    $Geo::GDAL::FFI::parent{$g} = $self;
+    Geo::GDAL::FFI::_register_parent_ref ($g, $self);
     $ref{$g} = 1;
     return bless \$g, 'Geo::GDAL::FFI::Geometry';
 }
@@ -400,6 +400,21 @@ sub IsRing {
     return Geo::GDAL::FFI::OGR_G_IsRing($$self);
 }
 
+sub GetEnvelope {
+    my ($self) = @_;
+    my $envelope = [0,0,0,0];
+    Geo::GDAL::FFI::OGR_G_GetEnvelope ($$self, $envelope);
+    return $envelope;
+}
+
+sub GetEnvelope3D {
+    my ($self) = @_;
+    my $envelope = [0,0,0,0,0,0];
+    Geo::GDAL::FFI::OGR_G_GetEnvelope3D ($$self, $envelope);
+    return $envelope;
+}
+
+
 1;
 
 =pod
@@ -601,6 +616,17 @@ described in GDAL documentation.
 
 =head2 IsRing
 
+=head2 GetEnvelope
+
+Returns a four element array reference containing
+[Xmin, Xmax, Ymin, Ymax].
+
+=head2 GetEnvelope3D
+
+Returns a six element array reference containing
+[Xmin, Xmax, Ymin, Ymax, Zmin, Zmax].
+
+
 =head1 LICENSE
 
 This software is released under the Artistic License. See
